@@ -24,25 +24,13 @@ public class KeyMatcherTest {
     private static final String KEY1 = "foo";
     private static final String KEY2 = "bar";
 
-    @Test
-    public void isCoveredByTest() {
-        AttributeMatcher attributeMatcher1 = new KeyMatcher(Arrays.asList(KEY1));
-        AttributeMatcher attributeMatcher2 = new KeyMatcher(Arrays.asList(KEY1));
-
-        Assert.assertTrue(attributeMatcher1.isCoveredBy(attributeMatcher1));
-        Assert.assertTrue(attributeMatcher1.isCoveredBy(attributeMatcher2));
-        Assert.assertTrue(attributeMatcher1.isCoveredBy(AnyMatcher.INSTANCE));
-
-        Assert.assertFalse(AnyMatcher.INSTANCE.isCoveredBy(attributeMatcher1));
-    }
-
     @Test
     public void matchesTest() {
         Tag tag1 = new Tag(KEY1, null);
         Tag tag2 = new Tag(KEY2, null);
         AttributeMatcher attributeMatcher = new KeyMatcher(Arrays.asList(KEY1));
 
-        Assert.assertTrue(attributeMatcher.matches(Arrays.asList(tag1)));
-        Assert.assertFalse(attributeMatcher.matches(Arrays.asList(tag2)));
+        Assert.assertTrue(attributeMatcher.matches(tag1));
+        Assert.assertFalse(attributeMatcher.matches(tag2));
     }
 }
