@@ -5,7 +5,7 @@
  * Copyright 2016 mikes222
  * Copyright 2016-2018 devemux86
  * Copyright 2017 Ludwig M Brinckmann
- * Copyright 2017 Gustl22
+ * Copyright 2017-2019 Gustl22
  *
  * This program is free software: you can redistribute it and/or modify it under the
  * terms of the GNU Lesser General Public License as published by the Free Software
@@ -56,7 +56,6 @@ import java.io.RandomAccessFile;
 import java.nio.ByteBuffer;
 import java.nio.charset.Charset;
 import java.util.ArrayList;
-import java.util.Arrays;
 import java.util.Iterator;
 import java.util.List;
 import java.util.Map;
@@ -188,8 +187,8 @@ public final class MapFileWriter {
                 boolean labelPosition = this.configuration.isLabelPosition();
                 if (!labelPosition) {
                     List<OSMTag> tags = this.configuration.getTagMapping().getWayTags(this.way.getTags().keySet());
-                    for (OSMTag tag : tags) {
-                        labelPosition = tag.isLabelPosition();
+                    for (int i = 0, n = tags.size(); i < n; i++) {
+                        labelPosition = tags.get(i).isLabelPosition();
                         if (labelPosition)
                             break;
                     }
@@ -266,7 +265,7 @@ public final class MapFileWriter {
 
     static final int MIN_TILE_BUFFER_SIZE = 0xF00000; // 15MB
 
-    static final int POI_DATA_BUFFER_SIZE = 0x100000; // 1MB
+    static final int POI_DATA_BUFFER_SIZE = 0xA00000; // 10MB
 
     static final int TILE_BUFFER_SIZE = 0xA00000; // 10MB
 
@@ -452,7 +451,7 @@ public final class MapFileWriter {
         int tagAmount = node.getTags() == null ? 0 : node.getTags().size();
         if (tagAmount > (1 << HALF_BYTE_SHIFT) - 1) {
             // See #971
-            LOGGER.info(Arrays.toString(node.getTags().keySet().toArray()) + "\n" + Arrays.toString(node.getTags().values().toArray()));
+            LOGGER.severe("Too many tags: " + node.toStringDetailed());
             throw new RuntimeException("more than 15 tags aren't supported");
         }
         return (byte) (layer << HALF_BYTE_SHIFT | (short) tagAmount);
@@ -494,7 +493,7 @@ public final class MapFileWriter {
         int tagAmount = way.getTags() == null ? 0 : way.getTags().size();
         if (tagAmount > (1 << HALF_BYTE_SHIFT) - 1) {
             // See #971
-            LOGGER.info(Arrays.toString(way.getTags().keySet().toArray()) + "\n" + Arrays.toString(way.getTags().values().toArray()));
+            LOGGER.severe("Too many tags: " + way.toStringDetailed());
             throw new RuntimeException("more than 15 tags aren't supported");
         }
         return (byte) (layer << HALF_BYTE_SHIFT | (short) tagAmount);
