@@ -1,5 +1,6 @@
 /*
- * Copyright 2017 usrusr
+ * Copyright 2017-2022 usrusr
+ * Copyright 2019 devemux86
  *
  * This program is free software: you can redistribute it and/or modify it under the
  * terms of the GNU Lesser General Public License as published by the Free Software
@@ -19,25 +20,24 @@ import org.mapsforge.core.graphics.GraphicFactory;
 import org.mapsforge.core.graphics.HillshadingBitmap;
 import org.mapsforge.core.model.BoundingBox;
 
-import java.io.File;
+import java.io.IOException;
 import java.lang.ref.WeakReference;
-import java.util.ArrayList;
-import java.util.HashMap;
-import java.util.Iterator;
-import java.util.LinkedHashSet;
-import java.util.List;
-import java.util.Map;
+import java.util.*;
 import java.util.concurrent.ExecutionException;
 import java.util.concurrent.Future;
+import java.util.logging.Logger;
 import java.util.regex.Matcher;
 import java.util.regex.Pattern;
-
+import java.util.zip.ZipEntry;
+import java.util.zip.ZipInputStream;
 
 /**
  * immutably configured, does the work for {@link MemoryCachingHgtReaderTileSource}
  */
 class HgtCache {
-    final File demFolder;
+    private static final Logger LOGGER = Logger.getLogger(HgtCache.class.getName());
+
+    final DemFolder demFolder;
     final boolean interpolatorOverlap;
     final ShadingAlgorithm algorithm;
     final int mainCacheSize;
@@ -51,7 +51,7 @@ class HgtCache {
     private LazyFuture<Map<TileKey, HgtFileInfo>> hgtFiles;
 
 
-    protected final static class TileKey {
+    protected static final class TileKey {
         final int north;
         final int east;
 
@@ -80,7 +80,7 @@ class HgtCache {
         }
     }
 
-    class Lru {
+    private static class Lru {
         public int getSize() {
             return size;
         }
@@ -92,7 +92,6 @@ class HgtCache {
             if (size < lru.size()) synchronized (lru) {
                 Iterator<Future<HillshadingBitmap>> iterator = lru.iterator();
                 while (lru.size() > size) {
-                    Future<HillshadingBitmap> evicted = iterator.next();
                     iterator.remove();
                 }
             }
@@ -138,7 +137,7 @@ class HgtCache {
 
     private List<String> problems = new ArrayList<>();
 
-    HgtCache(File demFolder, boolean interpolationOverlap, GraphicFactory graphicsFactory, ShadingAlgorithm algorithm, int mainCacheSize, int neighborCacheSize) {
+    HgtCache(DemFolder demFolder, boolean interpolationOverlap, GraphicFactory graphicsFactory, ShadingAlgorithm algorithm, int mainCacheSize, int neighborCacheSize) {
         this.demFolder = demFolder;
         this.interpolatorOverlap = interpolationOverlap;
         this.graphicsFactory = graphicsFactory;
@@ -151,47 +150,72 @@ class HgtCache {
 
         hgtFiles = new LazyFuture<Map<TileKey, HgtFileInfo>>() {
             @Override
-            protected Map<TileKey, HgtFileInfo> calculate() throws ExecutionException, InterruptedException {
+            protected Map<TileKey, HgtFileInfo> calculate() {
                 Map<TileKey, HgtFileInfo> map = new HashMap<>();
-                Matcher matcher = Pattern.compile("([ns])(\\d{1,2})([ew])(\\d{1,3})\\.hgt", Pattern.CASE_INSENSITIVE).matcher("");
+                Matcher matcher = Pattern.compile("([ns])(\\d{1,2})([ew])(\\d{1,3})\\.(?:(hgt)|(zip))", Pattern.CASE_INSENSITIVE).matcher("");
                 crawl(HgtCache.this.demFolder, matcher, map, problems);
                 return map;
             }
 
-            void crawl(File file, Matcher matcher, Map<TileKey, HgtFileInfo> map, List<String> problems) {
-                if (file.exists()) {
-                    if (file.isFile()) {
-                        String name = file.getName();
-                        if (matcher.reset(name).matches()) {
-                            int northsouth = Integer.parseInt(matcher.group(2));
-                            int eastwest = Integer.parseInt(matcher.group(4));
+            void crawl(DemFile file, Matcher matcher, Map<TileKey, HgtFileInfo> map, List<String> problems) {
+                String name = file.getName();
+                if (matcher.reset(name).matches()) {
+                    int northsouth = Integer.parseInt(matcher.group(2));
+                    int eastwest = Integer.parseInt(matcher.group(4));
 
-                            int north = "n".equals(matcher.group(1).toLowerCase()) ? northsouth : -northsouth;
-                            int east = "e".equals(matcher.group(3).toLowerCase()) ? eastwest : -eastwest;
+                    int north = "n".equals(matcher.group(1).toLowerCase()) ? northsouth : -northsouth;
+                    int east = "e".equals(matcher.group(3).toLowerCase()) ? eastwest : -eastwest;
 
-                            long length = file.length();
-                            long heights = length / 2;
-                            long sqrt = (long) Math.sqrt(heights);
-                            if (sqrt * sqrt != heights) {
-                                if (problems != null)
-                                    problems.add(file + " length in shorts (" + heights + ") is not a square number");
-                            } else {
-                                TileKey tileKey = new TileKey(north, east);
-                                HgtFileInfo existing = map.get(tileKey);
-                                if (existing == null || existing.size < length) {
-//                                hgtFiles.put(tileKey, new HgtFileInfo(file, east, north, east+1, north-1));
-                                    map.put(tileKey, new HgtFileInfo(file, north - 1, east, north, east + 1));
+                    long length = 0;
+
+                    if (matcher.group(6) == null) {
+                        length = file.getSize();
+                    } else {
+                        // zip
+                        ZipInputStream zipInputStream = null;
+                        try {
+                            zipInputStream = new ZipInputStream(file.openInputStream());
+                            String expectedHgt = name.toLowerCase().substring(0, name.length() - 4) + ".hgt";
+                            ZipEntry entry;
+                            while (null != (entry = zipInputStream.getNextEntry())) {
+                                if (expectedHgt.equals(entry.getName().toLowerCase())) {
+                                    length = entry.getSize();
+                                    break;
                                 }
                             }
+                        } catch (IOException e) {
+                            problems.add("could not read zip file " + file.getName());
                         }
-                    } else if (file.isDirectory()) {
-                        File[] files = file.listFiles();
-                        if (files != null) {
-                            for (File sub : files) {
-                                crawl(sub, matcher, map, problems);
+                        if (zipInputStream != null) {
+                            try {
+                                zipInputStream.close();
+                            } catch (IOException e) {
+                                e.printStackTrace();
                             }
                         }
                     }
+                    long heights = length / 2;
+                    long sqrt = (long) Math.sqrt(heights);
+                    if (heights == 0 || sqrt * sqrt != heights) {
+                        if (problems != null)
+                            problems.add(file + " length in shorts (" + heights + ") is not a square number");
+                        return;
+                    }
+
+                    TileKey tileKey = new TileKey(north, east);
+                    HgtFileInfo existing = map.get(tileKey);
+                    if (existing == null || existing.size < length) {
+                        map.put(tileKey, new HgtFileInfo(file, north - 1, east, north, east + 1, length));
+                    }
+                }
+            }
+
+            void crawl(DemFolder file, Matcher matcher, Map<TileKey, HgtFileInfo> map, List<String> problems) {
+                for (DemFile demFile : file.files()) {
+                    crawl(demFile, matcher, map, problems);
+                }
+                for (DemFolder sub : file.subs()) {
+                    crawl(sub, matcher, map, problems);
                 }
             }
         };
@@ -269,15 +293,15 @@ class HgtCache {
 
 
     class HgtFileInfo extends BoundingBox implements ShadingAlgorithm.RawHillTileSource {
-        final File file;
+        final DemFile file;
         WeakReference<Future<HillshadingBitmap>> weakRef = null;
 
         final long size;
 
-        HgtFileInfo(File file, double minLatitude, double minLongitude, double maxLatitude, double maxLongitude) {
+        HgtFileInfo(DemFile file, double minLatitude, double minLongitude, double maxLatitude, double maxLongitude, long size) {
             super(minLatitude, minLongitude, maxLatitude, maxLongitude);
             this.file = file;
-            size = file.length();
+            this.size = size;
         }
 
         Future<HillshadingBitmap> getBitmapFuture(double pxPerLat, double pxPerLng) {
@@ -389,7 +413,7 @@ class HgtCache {
         }
 
         @Override
-        public File getFile() {
+        public DemFile getFile() {
             return file;
         }
 
@@ -437,10 +461,11 @@ class HgtCache {
     }
 
 
-    public HillshadingBitmap getHillshadingBitmap(int northInt, int eastInt, double pxPerLat, double pxPerLng) throws InterruptedException, ExecutionException {
+    HillshadingBitmap getHillshadingBitmap(int northInt, int eastInt, double pxPerLat, double pxPerLng) throws InterruptedException, ExecutionException {
         HgtFileInfo hgtFileInfo = hgtFiles.get().get(new TileKey(northInt, eastInt));
-        if (hgtFileInfo == null)
+        if (hgtFileInfo == null) {
             return null;
+        }
 
         Future<HillshadingBitmap> future = hgtFileInfo.getBitmapFuture(pxPerLat, pxPerLng);
         return future.get();
@@ -454,25 +479,25 @@ class HgtCache {
             sink = center;
             source = neighbor;
             copyCanvas.setBitmap(sink);
-            copyCanvas.setClip(sink.getWidth() - padding, padding, padding, sink.getHeight() - 2 * padding);
+            copyCanvas.setClip(sink.getWidth() - padding, padding, padding, sink.getHeight() - 2 * padding, true);
             copyCanvas.drawBitmap(source, (source.getWidth() - 2 * padding), 0);
         } else if (border == HillshadingBitmap.Border.WEST) {
             sink = center;
             source = neighbor;
             copyCanvas.setBitmap(sink);
-            copyCanvas.setClip(0, padding, padding, sink.getHeight() - 2 * padding);
+            copyCanvas.setClip(0, padding, padding, sink.getHeight() - 2 * padding, true);
             copyCanvas.drawBitmap(source, 2 * padding - (source.getWidth()), 0);
         } else if (border == HillshadingBitmap.Border.NORTH) {
             sink = center;
             source = neighbor;
             copyCanvas.setBitmap(sink);
-            copyCanvas.setClip(padding, 0, sink.getWidth() - 2 * padding, padding);
+            copyCanvas.setClip(padding, 0, sink.getWidth() - 2 * padding, padding, true);
             copyCanvas.drawBitmap(source, 0, 2 * padding - (source.getHeight()));
         } else if (border == HillshadingBitmap.Border.SOUTH) {
             sink = center;
             source = neighbor;
             copyCanvas.setBitmap(sink);
-            copyCanvas.setClip(padding, sink.getHeight() - padding, sink.getWidth() - 2 * padding, padding);
+            copyCanvas.setClip(padding, sink.getHeight() - padding, sink.getWidth() - 2 * padding, padding, true);
             copyCanvas.drawBitmap(source, 0, (source.getHeight() - 2 * padding));
         }
     }
