@@ -1,7 +1,7 @@
 /*
  * Copyright 2010, 2011, 2012, 2013 mapsforge.org
  * Copyright 2014 Ludwig M Brinckmann
- * Copyright 2014-2016 devemux86
+ * Copyright 2014-2021 devemux86
  * Copyright 2014 Erik Duisters
  * Copyright 2014 Christian Pesch
  *
@@ -18,14 +18,7 @@
  */
 package org.mapsforge.map.scalebar;
 
-import org.mapsforge.core.graphics.Canvas;
-import org.mapsforge.core.graphics.Cap;
-import org.mapsforge.core.graphics.Color;
-import org.mapsforge.core.graphics.FontFamily;
-import org.mapsforge.core.graphics.FontStyle;
-import org.mapsforge.core.graphics.GraphicFactory;
-import org.mapsforge.core.graphics.Paint;
-import org.mapsforge.core.graphics.Style;
+import org.mapsforge.core.graphics.*;
 import org.mapsforge.map.model.DisplayModel;
 import org.mapsforge.map.model.IMapViewPosition;
 import org.mapsforge.map.model.MapViewDimension;
@@ -45,7 +38,6 @@ public class DefaultMapScaleBar extends MapScaleBar {
 
     public enum ScaleBarMode {BOTH, SINGLE}
 
-    private final float scale;
     private ScaleBarMode scaleBarMode;
     private DistanceUnitAdapter secondaryDistanceUnitAdapter;
 
@@ -61,12 +53,11 @@ public class DefaultMapScaleBar extends MapScaleBar {
 
     public DefaultMapScaleBar(IMapViewPosition mapViewPosition, MapViewDimension mapViewDimension,
                               GraphicFactory graphicFactory, DisplayModel displayModel, float scale) {
-        super(mapViewPosition, mapViewDimension, displayModel, graphicFactory, (int) (BITMAP_WIDTH * scale), (int) (BITMAP_HEIGHT * scale));
+        super(mapViewPosition, mapViewDimension, displayModel, graphicFactory, (int) (BITMAP_WIDTH * scale), (int) (BITMAP_HEIGHT * scale), scale);
 
         setMarginHorizontal((int) (DEFAULT_HORIZONTAL_MARGIN * scale));
         setMarginVertical((int) (DEFAULT_VERTICAL_MARGIN * scale));
 
-        this.scale = scale;
         this.scaleBarMode = ScaleBarMode.BOTH;
         this.secondaryDistanceUnitAdapter = ImperialUnitAdapter.INSTANCE;
 
@@ -76,6 +67,11 @@ public class DefaultMapScaleBar extends MapScaleBar {
         this.paintScaleTextStroke = createTextPaint(Color.WHITE, 2, Style.STROKE);
     }
 
+    public void setColor(int color) {
+        this.paintScaleBar.setColor(color);
+        this.paintScaleText.setColor(color);
+    }
+
     /**
      * @return the secondary {@link DistanceUnitAdapter} in use by this MapScaleBar
      */
@@ -110,7 +106,7 @@ public class DefaultMapScaleBar extends MapScaleBar {
         paint.setColor(color);
         paint.setStrokeWidth(strokeWidth * this.scale);
         paint.setStyle(style);
-        paint.setStrokeCap(Cap.SQUARE);
+        paint.setStrokeCap(Cap.ROUND);
         return paint;
     }
 
